﻿import sys
import itertools
import os

 #  properties.py:
 #      generates code that simply depends on the properties.
 #
 #  Remarks:
 #      This program doesn't read the table structure.  Properties are
 #      generated by tablegen.py, and this program reads its generated
 #      properties (code).
 #
 #      private int _userId; // 用户的 id

class Property(object):
    def __init__(self, type="", name=""):
        self.type = type
        self.name = name

    def parse(self, line):
        assert(not line.endswith("\n"))
        pos = line.find("//")
        if pos >= 0:
            line = line[:pos]
        line = line.strip()
        if line.startswith("private"):
            line = line[len("private"):].strip()
        elif line.startswith("public"):
            line = line[len("public"):].strip()
        # eliminate unwanted characters
        line = line.replace("\t", " ")
        while "  " in line:
            line = line.replace("  ", " ")
        line = line.replace(";", "")
        # linepts: line parts
        linepts = line.split(" ")
        self.type = linepts[0]
        self.name = linepts[1]

     #  get_colname: gets the column name corresponding to the internal variable
     #  name
    def get_colname(self):
        assert(len(self.name) >= 2)
        ch0 = self.name[0]
        assert(ch0 == "_")
        ch1 = self.name[1]
        return ch1.upper() + self.name[2:]

 #  input: the input lines must be without trailing "\n"
def read_properties(input):
    props = []
    for line in input:
        prop = Property()
        prop.parse(line)
        props.append(prop)
    return props

def main():
    props = read_properties([x.rstrip("\n") for x in sys.stdin])
    generate_tablestruct(sys.stdout, props)
    generate_props(sys.stdout, props)

def generate_tablestruct(file, props):
    writeline(file, "public DataTable GetTableStructure()")
    writeline(file, "{")
    writeline(file, "\tDataTable dt;")
    writeline(file, "")
    writeline(file, "\tdt = new DataTable();")
    for prop in props:
        indent = "\t"
        line = [indent, "dt.Columns.Add(\""]
        line.append(prop.get_colname())
        line.append("\", typeof(")
        line.append(prop.type)
        line.append("));")
        writeline(file, "".join(line))
    writeline(file, "\treturn dt;")
    writeline(file, "}")
    writeline(file, "")

def generate_props(file, props):
    for prop in props:
        line = ["public ", prop.type, " ", prop.get_colname()]
        writeline(file, "".join(line))
        writeline(file, "{")
        line = ["\tget { return ", prop.name, "; }"]
        writeline(file, "".join(line))
        line = ["\tset { ", prop.name, " = value; }"]
        writeline(file, "".join(line))
        writeline(file, "}")
        writeline(file, "")

def writeline(file, str):
    file.write(str)
    file.write("\n")

sys.exit(main())
